##initial read in and analysis of data, KEGG mapping metabolites and genes
options(stringsAsFactors = FALSE)
library(data.table)
library(ggplot2)
library(KEGGREST)

metabolite_file = "metabolitesAllReps.txt"
expr_file = "EcoliExpr_allNormalizedFromGEO.txt"
probe_info_file = "GPL10041_probeInfo.txt"

genes = fread(expr_file)

gene_info = fread(probe_info_file)
setnames(gene_info, "synonym", "Gene")
setnames(gene_info, "ID", "ID_REF")
all_ecoli_genes = keggLink(target="eco", source="ko")
all_ecoli_genes = gsub("eco:","", all_ecoli_genes)
names(all_ecoli_genes) = gsub("ko:", "", names(all_ecoli_genes))
all_ecoli_genes = data.table(KO=names(all_ecoli_genes), Gene=all_ecoli_genes)

#check on ethanolamine enzymes
#all_ecoli_genes[Gene %in% c("b2239","b3449")] #K01126
# ecoli_kos = fread("EcoliExprKO.txt")
# genes2 = merge(genes, gene_info[,list(ID_REF, Gene)], by="ID_REF")
# qplot(as.numeric(unlist(genes2[Gene=="b2239"])), as.numeric(unlist(genes2[Gene=="b3449"])))
# qplot(as.numeric(unlist(genes2[Gene=="b2239"]))[2:(ncol(genes2)-1)], as.numeric(unlist(ecoli_kos[KO=="K01126"]))[2:ncol(ecoli_kos)])
# qplot(as.numeric(unlist(genes2[Gene=="b3449"]))[2:(ncol(genes2)-1)], as.numeric(unlist(ecoli_kos[KO=="K01126"]))[2:ncol(ecoli_kos)])
#basically entirely from b2239 - periplasmic glpQ

gene_info = merge(gene_info, all_ecoli_genes, by="Gene", all=T, allow.cartesian=T)

genes2 = merge(genes, gene_info[,list(ID_REF, KO)], all.x=T, all.y=F, by="ID_REF",allow.cartesian=T)
genes2 = genes2[!is.na(KO)]
setkey(genes2,KO)
non_unique= unique(genes2[,KO][duplicated(genes2[,KO])]) # check for repetition
genes3 = genes2
genes3[,ID_REF:=NULL]
setkey(genes3, NULL)
genes3 = unique(genes3)
genes3 = genes3[,lapply(.SD, sum), by="KO"]

write.table(genes3, file = "EcoliExprKO.txt", quote=F, row.names=F, sep = "\t")

expr_ids = data.table(ID=names(genes3)[names(genes3)!="KO"])
expr_ids[,Stress:=regmatches(ID, regexpr("coli_[a-z]*", ID))]
expr_ids[,Stress:=gsub("coli_","",Stress)]
expr_ids[,TimePoint:=regmatches(ID, regexpr("timepoint[0-9]",ID))]
expr_ids[,TimePoint:=gsub("timepoint","", TimePoint)]
expr_ids[,Rep:=gsub("coli_[a-z]*_timepoint[0-9]+_rep","",ID)]

#I think the biological replicates correspond between the genes and mets, so we'll use that
#"All data were normalized to cell number and the chromatographic internal standard."
mets = fread(metabolite_file)
setnames(mets, c("Sample", "Stress", "Timepoint", "TechnicalRep", "Rep", "OpDensity", "InternalStd", "SampNorm", names(mets)[9:204]))
#Normalize with respect to internal standard
mets2 = data.table(mets[,1:8,with=F], apply(mets[,9:204,with=F], 2, function(x){ x/mets[,InternalStd]}))
#Normalize by cell number (proportional to Optical Density)
mets2 = data.table(mets2[,1:8,with=F], apply(mets2[,9:204,with=F], 2, function(x){ x/mets2[,OpDensity]}))

met_names = names(mets2)[9:204]
good_met_names = met_names[!grepl("unknown|similar", met_names)]
met_ids = sapply(good_met_names, keggFind, database="compound")
kegg_ids = rep(NA, length(good_met_names))

for(j in 1:length(good_met_names)){
  if(length(met_ids[[j]])!=0) kegg_ids[j] = gsub("cpd:","", names(met_ids[[j]])[1])
}

#check by hand
met_ids[[4]] = keggFind("glycine",database="compound")
kegg_ids[4] = gsub("cpd:","", names(met_ids[[4]])[1])
kegg_ids[7] = names(met_ids[[7]])[2]
kegg_ids[8] = NA #unclear what the correct match is
kegg_ids[9] = names(met_ids[[9]])[2]
kegg_ids[10] = names(met_ids[[10]])[2]
kegg_ids[15] = names(met_ids[[15]])[5]
kegg_ids[18] = names(met_ids[[18]])[3]
met_ids[[19]] = keggFind("serine", database="compound")
kegg_ids[19] = gsub("cpd:","",names(met_ids[[19]])[1])
kegg_ids[20] = names(met_ids[[20]])[2]
met_ids[[24]] = keggFind("uracil", database="compound")
kegg_ids[24] = gsub("cpd:", "", names(met_ids[[24]])[1])
met_ids[[31]] = keggFind("threonate", database="compound")
kegg_ids[31] = names(met_ids[[31]])[1]
kegg_ids[32] = names(met_ids[[32]])[6]
kegg_ids[33] = names(met_ids[[33]])[2]
met_ids[[34]] = keggFind("compound","2-hydroxyglutarate")
kegg_ids[34] = names(met_ids[[34]])[2]
met_ids[[35]] = keggFind("compound","glutamate N-acetyl")
kegg_ids[35] = names(met_ids[[35]])[1]
kegg_ids[36] = NA
met_ids[[37]] = keggFind("compound", "pyroglutamate")
kegg_ids[37] = names(met_ids[[37]])[1]
met_ids[[41]] = keggFind("compound", "4-hydroxybenzoate")
kegg_ids[41] = names(met_ids[[41]])[1]
kegg_ids[42] = NA
met_ids[[43]] = keggFind("compound", "pyrophosphate")
kegg_ids[43] = names(met_ids[[43]])[1]
met_ids[[50]] = keggFind("compound","glycerate 3-phosphate")
kegg_ids[50] = names(met_ids[[50]])[1]
met_ids[[53]] = keggFind("compound", "galactonate")
kegg_ids[53] = names(met_ids[[53]])[1]
kegg_ids[54] = names(met_ids[[54]])[2]
kegg_ids[62] = names(met_ids[[62]])[2]
met_ids[[63]] = keggFind("compound", "lactitol")
kegg_ids[63] = names(met_ids[[63]])[1]
met_ids[[68]] = keggFind("compound", "phosphoenolpyruvate")
kegg_ids[68] = names(met_ids[[68]])[1]
met_ids[[73]] = keggFind("compound", "D-erythrose 4-phosphate")
kegg_ids[73] = names(met_ids[[73]])[1]
met_ids[[79]] = keggFind("compound", "glutamine")
kegg_ids[79] = names(met_ids[[79]])[1]
kegg_ids[81] = names(met_ids[[81]])[2]
kegg_ids[83]= names(met_ids[[83]])[2]
met_ids[[85]] = keggFind("compound", "asparagine")
kegg_ids[85] = names(met_ids[[85]])[1]
kegg_ids[88] = names(met_ids[[88]])[2]
met_ids[[90]] = keggFind("compound", "shikimate 3-phosphate")
kegg_ids[90] = names(met_ids[[90]])[1]
met_ids[[96]] = keggFind("compound", "ribose 5-phosphate")
kegg_ids[96] = names(met_ids[[96]])[1]
met_ids[[97]] = keggFind("compound", "spermidine")
kegg_ids[97] = names(met_ids[[97]])[1]
kegg_ids[99] = "C00345"
met_ids[[100]] = keggFind("compound", "linolenate")
kegg_ids[100] = names(met_ids[[100]])[1]

kegg_ids = gsub("cpd:", "", kegg_ids)

good_kegg_ids = kegg_ids[!grepl("minor", good_met_names) & !is.na(kegg_ids)] #better to do one more filter, I think
good_met_names = good_met_names[!grepl("minor", good_met_names) & !is.na(kegg_ids)]

mets_sub = mets2[,c(rep(T, 8), (names(mets2)[9:204] %in% good_met_names)),with=F]
setnames(mets_sub, c(names(mets_sub)[1:8], good_kegg_ids))

##For the one duplicated ID both sets of values are very highly correlated so we'll just add them together
mets_sub[,C00009:=NULL]
mets_sub[,C00009:=mets2[,` phosphoric acid`] + mets2[,`Pyrophosphate (cont?)`]]

##Median over technical replicates
mets_sub[,Sample:=NULL]
mets_sub[,InternalStd:=NULL]
mets_sub[,TechnicalRep:=NULL]
mets_sub2 = mets_sub[,lapply(.SD,median,na.rm=T),by=c("Stress","Timepoint","Rep","OpDensity","SampNorm" )]



median_mets = fread("medianMetabolites_fig1.txt")

mets_sub2[,Stress:=gsub(" ","", Stress)]
mets_sub2[,Stress:=ifelse(Stress=="lactoseshift","lactose",Stress)]
mets_sub2[,ID:=paste0("coli_",Stress, "_timepoint", Timepoint, "_rep", Rep)]
mets_sub2[,PostPerturb:=ifelse((Timepoint > 2 & Stress!="lactose" & Stress!="control")|(Timepoint > 3 & Stress!="control"),1,0)]

metadata = unique(mets_sub2[,list(ID, Stress,Timepoint,Rep,OpDensity,SampNorm,PostPerturb)])
write.table(metadata, file = "mets_metadata.txt", sep = "\t", quote=F, row.names=F)
#we're just going to go with the replicates as they are labeled and will assess later

mets_sub3 = data.table(t(mets_sub2[,6:92,with=F]))
setnames(mets_sub3, mets_sub2[,ID])
mets_sub3[,KEGG:=names(mets_sub2)[6:92]]
setkey(mets_sub3, KEGG)
write.table(mets_sub3, file = "Ecoli_metsKeggGood.txt", quote=F, row.names=F, sep = "\t")

good_samps = intersect(names(mets_sub3), names(genes3))
genes3_good = genes3[,c("KO", good_samps), with=F]
mets3_good = mets_sub3[,c(good_samps, "KEGG"), with=F]
write.table(genes3_good, file = "genes_sub_good.txt", quote=F, row.names=F, sep = "\t")
write.table(mets3_good, file = "mets_sub_good.txt", quote=F, row.names=F, sep = "\t")

samples_sub_sub = metadata[(Timepoint==1 | Timepoint==4) & (ID %in% good_samps)]
genes3_sub = genes3[,c("KO", samples_sub_sub[,ID]),with=F]
mets3_sub = mets_sub3[,c(samples_sub_sub[,ID], "KEGG"),with=F]
write.table(genes3_sub, file = "genes_sub_tp14_good.txt", quote=F, row.names=F, sep = "\t")
write.table(mets3_sub, file = "mets_sub_tp14_good.txt", quote=F, row.names=F, sep = "\t")

# 
# ###Correlation matrices
# library(gplots)
# library(RColorBrewer)
# heatmap.2(cor(t(mets3_good[,good_samps,with=F]), use="complete.obs"), trace = "none", dendrogram="none", col = brewer.pal(9,"BuPu")) 
# heatmap.2(cor(t(mets3_good[,good_samps[grepl("control", good_samps)],with=F]), use="complete.obs"), trace = "none", dendrogram="none", col = brewer.pal(9,"BuPu")) 
# 
# library(CCA)
# library(vegan)
# mets_noNA = mets3_good[apply(mets3_good, 1,function(x){ !any(is.na(x))})]
# cc_test = cc(X=t(genes3_good[,good_samps,with=F]), Y=t(mets_noNA[,good_samps,with=F]))
# cc_test = CCorA(Y=t(genes3_good[,good_samps,with=F]), X=t(mets_noNA[,good_samps,with=F]))
