#metabolite ID processing for Swedish data
options(stringsAsFactors = FALSE)
library(data.table)

metabosearch_file = "Curtis_swedish/metabosearch_KEGG_noHMDB.txt"

all_ids = fread(metabosearch_file)
all_ids = unique(all_ids)
setkey(all_ids, Query_MZ)

all_mets_wide = dcast.data.table(all_ids, Query_MZ~KEGG)
masses = unique(all_ids[,Query_MZ])

all_ids_reduced = data.table()
#first, we're only going to take combos with the lowest Delta
for(j in 1:length(masses)){
  ids_sub = all_ids[Query_MZ==masses[j]]
  min_delt = min(ids_sub[,Delta])
  ids_sub2 = ids_sub[Delta ==min_delt]
  all_ids_reduced = rbind(all_ids_reduced, ids_sub2)
}


all_ids_expand = data.frame(Query_MZ=masses, Delta = rep(0, length(masses)), KEGG1 = rep('', length(masses)), KEGG2 = rep('', length(masses)),
                            KEGG3 = rep('', length(masses)), KEGG4 = rep('', length(masses)), KEGG5 = rep('', length(masses)), KEGG6 = rep('', length(masses)))

max_possible = 6
for(j in 1:length(masses)){
  possible_ids = all_ids[J(masses[j])][,KEGG]
  if(length(possible_ids)< max_possible){
    possible_ids = c(possible_ids, rep(NA, 6-length(possible_ids)))
  }
  all_ids_expand[j,2:8]=c(all_ids[J(masses[j])][,Delta][1], possible_ids[1:6])
}

write.table(all_ids_expand, file = "Curtis_swedish/met_ID_table.txt", sep = "\t", quote=F, row.names=F)

met_data = fread("Curtis_swedish/metabolites.txt")
met_data = met_data[c(-1,-2)]
setnames(met_data,"Mass Negative mode","Mass")

met_data = met_data[Mass %in% masses]

metadata = fread("Curtis_swedish/map-2.txt")
setnames(met_data, names(met_data), toupper(names(met_data)))

gene_data = fread("Curtis_swedish/highestGenes.tsv")
setnames(gene_data, names(gene_data), gsub("_14\\.highestGenes","",names(gene_data)))


#setnames(gene_data, names(gene_data)[2:length(names(gene_data))], 
 #        metadata[match(names(gene_data)[2:length(names(gene_data))],metadata[,`#SampleID`])][,PatientId])

bad_columns = metadata[`#SampleID` %in% names(gene_data)]
bad_columns = bad_columns[!is.na(PatientId) & PatientId!=""]
setkey(bad_columns, PatientId)
samps = unique(bad_columns[,PatientId])
for(j in 1:length(samps)){
  samp_ids = bad_columns[samps[j]][,`#SampleID`]
  if(length(samp_ids)>1 & all(samp_ids %in% names(gene_data))){
    gene_data[,samps[j]:=rowMeans(gene_data[,samp_ids,with=F])]
  }else{
    setnames(gene_data, samp_ids, samps[j])
  }
  for(k in 1:length(samp_ids)) gene_data[,samp_ids[k]:=NULL]
}
gene_data[,SRR504939:=NULL]

met_data_sub = met_data[,names(met_data) %in% c("MASS",names(gene_data)),with=F]

sorted_names = sort(intersect(names(met_data_sub),names(gene_data)))
gene_data = gene_data[,c("KO",sorted_names),with=F]
met_data_sub = met_data_sub[,c(sorted_names,"MASS"),with=F]

write.table(gene_data,file = "Curtis_swedish/genes_good.txt", sep = "\t", row.names=F, quote=F)
write.table(met_data_sub,file = "Curtis_swedish/met_data_sub.txt", sep = "\t", row.names=F, quote=F)



###Validation check - how many KOs present in each sample?
gene_data = fread("Curtis_swedish/genes_good.txt")
apply(gene_data[,2:13,with=F],2, function(x){length(x[x!=0])})
