#single species contributions picrust
#make_unnormalized = get formatted KO abundances for one species
#prmts = get single species PRMT
#contributions - do contributions analysis for all species
##Revision - aggregate to genus level 11/24/2015


options(stringsAsFactors = F)
library(data.table)
library(reshape2)
library(ggplot2)
library(getopt)
source("core_functions.R")

args = commandArgs(trailingOnly=T)

contrib_file = args[1] #"CD_mice_data/mice_metagenome_contributions.txt" #
data_dir = args[2] #"CD_mice_data/"
results_file = args[3] #"runs_nz4_noMP/mice_AB_out.rda"
out_dir = args[4]
otu_id = args[5]

#contrib_file = "/Users/cecilianoecker/Google Drive/Genome_Sciences/2014_Summer/rotation_pipeline/BV_data/bv_val_picrust_metagenome_contributions.txt"
#data_dir = "BV_data/"
#results_file = "runs_nz4_noMP/bv_val_picrust_out.rda"

cat(paste(otu_id, "\n"))
cat(paste(contrib_file, "\n"))
cat(paste(data_dir, "\n"))

if("make_unnormalized" %in% args){
  contribs = fread(contrib_file)
  all_samps = unique(contribs[,Sample])
  contribs[,RelAbundSample:=CountContributedByOTU/sum(CountContributedByOTU),by=Sample] #relative abundance of this gene from this taxon in all the genes
  if(otu_id!="all"){
    contribs = contribs[OTU==otu_id]  
    contribs = dcast.data.table(contribs, Gene~Sample, value.var = "RelAbundSample")
    #contribs = dcast.data.table(contribs, Gene~Sample, value.var = "ContributionPercentOfSample")
    for(j in 1:length(all_samps)){
      if(!(all_samps[j] %in% names(contribs))){
        contribs[,eval(all_samps[j]):=rep(0)]
      }
    }
    setnames(contribs, "Gene","KO")
    contribs = contribs[,c("KO",sort(all_samps)),with=F]
    write.table(contribs, file = paste0(out_dir,otu_id,"_KOabund.txt"), quote=F, row.names=F, sep = "\t")    
  }else{
    if("sum_to_genus" %in% args){
      cat("Summing to genus level, need taxonomy file\n")
      taxonomy = fread("97_otu_taxonomy.txt")
      taxonomy[,Genus:=gsub("; s__.*", "", V2)]
      setnames(taxonomy, c("OTU", "Taxonomy", "Genus"))
      contribs = merge(contribs, taxonomy, by="OTU", all.x=T, all.y=F)
      contribs = contribs[,sum(RelAbundSample), by=list(Gene, Sample, Genus)]
      setnames(contribs, "Genus", "OTU") #now just treat genera like otus
      setnames(contribs, "V1", "RelAbundSample")
    }
    all_otus = unique(contribs[,OTU])
    cat(all_otus)
    all_koAbunds_byOTU = vector("list", length(all_otus))
    for(k in 1:length(all_otus)){
      contribs_sub = contribs[OTU==all_otus[k]]  
      contribs_sub = dcast.data.table(contribs_sub, Gene~Sample, value.var = "RelAbundSample")
      #contribs_sub = dcast.data.table(contribs_sub, Gene~Sample, value.var = "ContributionPercentOfSample")
      for(j in 1:length(all_samps)){
        if(!(all_samps[j] %in% names(contribs_sub))){
          contribs_sub[,eval(all_samps[j]):=rep(0)]
        }
      }
      setnames(contribs_sub, "Gene","KO")
      all_koAbunds_byOTU[[k]] = contribs_sub[,c("KO",sort(all_samps)),with=F]
    }
    if("save" %in% args) save(all_koAbunds_byOTU, file = paste0(out_dir,"all_koAbunds_byOTU.rda"))
  }
}

if("prmts" %in% args){
  if(otu_id != "all"){
    kos_alone = fread(paste0(data_dir, otu_id, "_KOabund.txt"))
    #kos_alone = fread(paste0(data_dir,otu_id, "_KOabund_musicc.txt"))
    kos_alone = kos_alone[,c(names(kos_alone)[names(kos_alone)!="KO"],"KO"),with=F]
    #load("runs_nz4_noMP/bv_val_picrust_out.rda")
    load(results_file)
    #Best just to regenerate network
    sub_ko_net = generate_genomic_network(kos_alone[,KO], keggSource = "labKegg",degree_filter = 30)
    prmts_alone = get_prmt_scores(sub_ko_net[[1]], kos_alone)
    write.table(prmts_alone, file = paste0(data_dir, otu_id, "_prmts.txt"), quote=F, row.names=F, sep = "\t")
  }else{
    if("load" %in% args) load(paste0(out_dir, "all_koAbunds_byOTU.rda"))
    prmts_alone = vector("list", length(all_otus))
    for(k in 1:length(all_otus)){
      sub_ko_net = generate_genomic_network(all_koAbunds_byOTU[[k]][,KO], keggSource = "labKegg", degree_filter = 30)
      prmts_alone[[k]] = get_prmt_scores(sub_ko_net[[1]], all_koAbunds_byOTU[[k]])
    }
    if("save" %in% args) save(prmts_alone, file = paste0(out_dir, "all_prmtsAlone_byOTU.rda"))
  }
}

if(("contributions" %in% args) & otu_id=="all"){
  load(results_file)
  subjects = names(norm_kos)[names(norm_kos)!="KO"]
  prmts_sub_good = get_prmt_scores(ko_net[[1]], norm_kos)
  prmt_sub_good = prmts_sub_good[node_data[,compound]]
  all_rxns = lapply(node_data[,compound], function(x){ return(ko_net[[3]][Reac==x|Prod==x])})
  all_rxns = lapply(all_rxns,get_non_rev_rxns)
  if("load" %in% args){
    load(paste0(out_dir, "all_prmtsAlone_byOTU.rda"))
    contribs = fread(contrib_file)
    all_otus = unique(contribs[,OTU])
  } 
#   all_spec_prmt_files = list.files(path = data_dir, pattern = "_prmts.txt")
#   all_spec_prmts = vector("list", length(all_spec_prmt_files))
#   all_spec = gsub(all_spec_prmt_files, "_prmts.txt","")
#   for(j in 1:length(all_spec_prmts)) all_spec_prmts[[j]] = fread(paste0(data_dir, all_spec_prmt_files[j]))
#for mice only - subject id change
  for(j in 1:length(prmts_alone)){
    setnames(prmts_alone[[j]], gsub("NoAbx6wkrec","NoAbx6wk", names(prmts_alone[[j]])))
  }
  spec_contrib=lapply(1:length(node_data[,compound]),prmt_species_contributions_picrust, prmts_sub_good = prmt_sub_good, 
                      all_rxns = all_rxns, subjects = subjects, norm_kos = norm_kos, ko_net = ko_net, all_taxa = all_otus, 
                      single_spec_prmts = prmts_alone, cor_with=T)
  
  save(spec_contrib, file = paste0(out_dir, "picrust_spec_contrib.rda"))  
  
}
